unless defined?(Motion::Project::Config)
  raise "This file must be required within a RubyMotion project Rakefile."
end

if Motion::Project::App.template != :android
  raise "This file must be required within a RubyMotion Android project."
end

module Motion::Project
  class Config
    variable :maven

    def maven(&block)
      @maven ||= Motion::Project::Maven.new(self)
      if block
        @maven.instance_eval(&block)
      end
      @maven
    end
  end

  class Maven
    MAVEN_ROOT = 'vendor/Maven'

    def initialize(config)
      @maven_path = 'mvn'
      @config = config
      @dependencies = []
      @repositories = []
      configure_project
    end

    def configure_project
      @config.vendor_project(:jar => "#{MAVEN_ROOT}/target/dependencies.jar")
    end

    def path=(path)
      @maven_path = path
    end

    def dependency(name, options = {})
      @dependencies << normalized_dependency(name, options)
    end

    def repository(url, options = {})
      @repositories << normalized_repository(url, options)
    end

    def install!(update)
      generate_pom
      system "#{maven_command} -f #{pom_path} clean install"
    end

    # Helpers
    def generate_pom
      template_path = File.expand_path("../pom.erb", __FILE__)
      template = ERB.new(File.new(template_path).read, nil, "%")
      File.open(pom_path, 'w') do |io|
        io.puts template.result(binding)
      end
      system "xmllint --output #{pom_path} --format #{pom_path}"
    end

    def pom_path
      "#{MAVEN_ROOT}/pom.xml"
    end

    def maven_command
      unless system("command -v #{@maven_path} >/dev/null")
        $stderr.puts "[!] #{@maven_path} command doesn’t exist. Verify your maven installation."
        exit 1
      end

      if ENV['MAVEN_DEBUG']
        "#{@maven_path} -X"
      else
        "#{@maven_path}"
      end
    end

    def normalized_repository(url, options)
      {
        url: url,
        releases: options.fetch(:releases, true),
        snapshots: options.fetch(:snapshots, true)
      }
    end

    def normalized_dependency(name, options)
      {
        name: name,
        version: options.fetch(:version, 'LATEST'),
        artifact: options.fetch(:artifact, name),
        scope: options.fetch(:scope, false),
        type: options.fetch(:type, false)
      }
    end

    def inspect
      @dependencies.map do |dependency|
        "#{dependency[:name]} - #{dependency[:artifact]} (#{dependency[:version]})"
      end.inspect
    end
  end
end

namespace :maven do
  desc "Download and build dependencies"
  task :install do
    FileUtils.mkdir_p Motion::Project::Maven::MAVEN_ROOT
    dependencies = App.config.maven
    dependencies.install!(true)
  end
end

namespace :clean do
  # This gets appended to the already existing clean:all task.
  task :all do
    dir = Motion::Project::Maven::MAVEN_ROOT
    if File.exist?(dir)
      App.info 'Delete', dir
      rm_rf dir
    end
  end
end
